@@ -648,8 +648,10 @@ test_close_cancel (void)
 {
   test_data_t *test = setup_test ();
 
-  test_open_both_connections (test);
+  wocky_test_stream_set_write_mode (test->stream->stream0_output,
+    WOCKY_TEST_STREAM_WRITE_COMPLETE);
 
+  test_open_both_connections (test);
   wocky_porter_start (test->sched_out);
 
   wocky_xmpp_connection_recv_stanza_async (test->in, NULL,
@@ -2352,6 +2354,9 @@ test_close_error (void)
 {
   test_data_t *test = setup_test ();
 
+  wocky_test_stream_set_write_mode (test->stream->stream0_output,
+    WOCKY_TEST_STREAM_WRITE_COMPLETE);
+
   test_open_both_connections (test);
   wocky_porter_start (test->sched_in);
 
@@ -2552,6 +2557,9 @@ test_close_force (void)
   test_data_t *test = setup_test ();
   WockyStanza *s;
 
+  wocky_test_stream_set_write_mode (test->stream->stream0_output,
+    WOCKY_TEST_STREAM_WRITE_COMPLETE);
+
   test_open_both_connections (test);
   wocky_porter_start (test->sched_in);
 
@@ -3119,6 +3127,169 @@ handler_from_anyone (void)
   teardown_test (test);
 }
 
+static void
+closed_cb (GObject *source,
+    GAsyncResult *result,
+    gpointer user_data)
+{
+  WockyPorter *porter = WOCKY_PORTER (source);
+  test_data_t *test = user_data;
+  gboolean ret;
+  GError *error = NULL;
+
+  ret = wocky_porter_close_finish (porter, result, &error);
+  g_assert_no_error (error);
+  g_assert (ret);
+
+  test->outstanding--;
+  g_main_loop_quit (test->loop);
+}
+
+static void
+sent_stanza_cb (GObject *source,
+    GAsyncResult *result,
+    gpointer user_data)
+{
+  WockyPorter *porter = WOCKY_PORTER (source);
+  test_data_t *test = user_data;
+  gboolean ret;
+  GError *error = NULL;
+
+  ret = wocky_porter_send_finish (porter, result, &error);
+  g_assert_no_error (error);
+  g_assert (ret);
+
+  /* Close up both porters. There's no reason why either of these operations
+   * should fail.
+   */
+  wocky_porter_close_async (test->sched_out, NULL, closed_cb, test);
+  wocky_porter_close_async (test->sched_in, NULL, closed_cb, test);
+}
+
+static void
+close_from_send_callback (void)
+{
+  test_data_t *test = setup_test ();
+  WockyStanza *stanza = wocky_stanza_build (WOCKY_STANZA_TYPE_MESSAGE,
+      WOCKY_STANZA_SUB_TYPE_NONE, NULL, NULL,
+      '(', "body", '$', "I am made of chalk.", ')', NULL);
+
+  /* Fire up porters in both directions. */
+  test_open_both_connections (test);
+  wocky_porter_start (test->sched_in);
+  wocky_porter_start (test->sched_out);
+
+  /* Send a stanza. Once it's been safely sent, we should be able to close up
+   * the connection in both directions without any trouble.
+   */
+  wocky_porter_send_async (test->sched_in, stanza, NULL, sent_stanza_cb, test);
+  g_object_unref (stanza);
+
+  /* The two outstanding events are both porters ultimately closing
+   * successfully. */
+  test->outstanding += 2;
+  test_wait_pending (test);
+
+  teardown_test (test);
+}
+
+/* Callbacks used in send_from_send_callback() */
+static gboolean
+message_received_cb (
+    WockyPorter *porter,
+    WockyStanza *stanza,
+    gpointer user_data)
+{
+  test_data_t *test = user_data;
+
+  test_expected_stanza_received (test, stanza);
+  return TRUE;
+}
+
+static void
+sent_second_or_third_stanza_cb (
+    GObject *source,
+    GAsyncResult *result,
+    gpointer user_data)
+{
+  test_data_t *test = user_data;
+  GError *error = NULL;
+
+  wocky_porter_send_finish (WOCKY_PORTER (source), result, &error);
+  g_assert_no_error (error);
+
+  test->outstanding--;
+  g_main_loop_quit (test->loop);
+}
+
+static void
+sent_first_stanza_cb (
+    GObject *source,
+    GAsyncResult *result,
+    gpointer user_data)
+{
+  test_data_t *test = user_data;
+  WockyStanza *third_stanza;
+  GError *error = NULL;
+
+  wocky_porter_send_finish (WOCKY_PORTER (source), result, &error);
+  g_assert_no_error (error);
+
+  third_stanza = wocky_stanza_build (WOCKY_STANZA_TYPE_MESSAGE,
+      WOCKY_STANZA_SUB_TYPE_NONE, NULL, NULL,
+      '(', "body", '$', "I am made of dur butter.", ')', NULL);
+  wocky_porter_send_async (test->sched_in, third_stanza, NULL,
+      sent_second_or_third_stanza_cb, test);
+  g_queue_push_tail (test->expected_stanzas, third_stanza);
+  /* One for the callback; one for the receiving end. */
+  test->outstanding += 2;
+
+  test->outstanding--;
+  g_main_loop_quit (test->loop);
+}
+
+static void
+send_from_send_callback (void)
+{
+  test_data_t *test = setup_test ();
+  WockyStanza *stanza;
+
+  test_open_both_connections (test);
+  wocky_porter_start (test->sched_in);
+  wocky_porter_start (test->sched_out);
+
+  wocky_porter_register_handler_from_anyone (test->sched_out,
+      WOCKY_STANZA_TYPE_MESSAGE, WOCKY_STANZA_SUB_TYPE_NONE,
+      WOCKY_PORTER_HANDLER_PRIORITY_NORMAL, message_received_cb, test,
+      '(', "body", ')', NULL);
+
+  /* Send a stanza; in the callback for this stanza, we'll send another stanza.
+   */
+  stanza = wocky_stanza_build (WOCKY_STANZA_TYPE_MESSAGE,
+      WOCKY_STANZA_SUB_TYPE_NONE, NULL, NULL,
+      '(', "body", '$', "I am made of chalk.", ')', NULL);
+  wocky_porter_send_async (test->sched_in, stanza, NULL,
+      sent_first_stanza_cb, test);
+  g_queue_push_tail (test->expected_stanzas, stanza);
+  /* One for the callback; one for the receiving end. */
+  test->outstanding += 2;
+
+  /* But before we've had a chance to send that one, send a second. */
+  stanza = wocky_stanza_build (WOCKY_STANZA_TYPE_MESSAGE,
+      WOCKY_STANZA_SUB_TYPE_NONE, NULL, NULL,
+      '(', "body", '$', "I am made of jelly.", ')', NULL);
+  wocky_porter_send_async (test->sched_in, stanza, NULL,
+      sent_second_or_third_stanza_cb, test);
+  g_queue_push_tail (test->expected_stanzas, stanza);
+  /* One for the callback; one for the receiving end. */
+  test->outstanding += 2;
+
+  test_wait_pending (test);
+
+  test_close_both_porters (test);
+  teardown_test (test);
+}
+
 int
 main (int argc, char **argv)
 {
@@ -3177,6 +3348,10 @@ main (int argc, char **argv)
   g_test_add_func ("/xmpp-porter/send-and-disconnect", send_and_disconnect);
   g_test_add_func ("/xmpp-porter/handler-for-domain", handler_for_domain);
   g_test_add_func ("/xmpp-porter/handler-from-anyone", handler_from_anyone);
+  g_test_add_func ("/xmpp-porter/close-from-send-callback",
+      close_from_send_callback);
+  g_test_add_func ("/xmpp-porter/send-from-send-callback",
+      send_from_send_callback);
 
   result = g_test_run ();
   test_deinit ();
