@@ -30,8 +30,8 @@ setup_test_full (guint timeout,
   data->in = wocky_xmpp_connection_new (data->stream->stream0);
   data->out = wocky_xmpp_connection_new (data->stream->stream1);
 
-  data->session_in = wocky_session_new (data->in, in_jid);
-  data->session_out = wocky_session_new (data->out, out_jid);
+  data->session_in = wocky_session_new_with_connection (data->in, in_jid);
+  data->session_out = wocky_session_new_with_connection (data->out, out_jid);
 
   data->sched_in = wocky_session_get_porter (data->session_in);
   data->sched_out = wocky_session_get_porter (data->session_out);
