@@ -24,12 +24,10 @@
 #define GABBLE_CAPS_CHANNEL_MANAGER_H
 
 #include <glib-object.h>
-#include <loudmouth/loudmouth.h>
 #include <telepathy-glib/exportable-channel.h>
 #include <telepathy-glib/handle.h>
 
 #include "capabilities.h"
-#include "types.h"
 
 G_BEGIN_DECLS
 
@@ -46,11 +44,10 @@ G_BEGIN_DECLS
 
 #define GABBLE_CAPS_CHANNEL_MANAGER_GET_INTERFACE(obj) \
   (G_TYPE_INSTANCE_GET_INTERFACE ((obj), \
-  GABBLE_TYPE_CAPS_CHANNEL_MANAGER, GabbleCapsChannelManagerIface))
+  GABBLE_TYPE_CAPS_CHANNEL_MANAGER, GabbleCapsChannelManagerInterface))
 
 typedef struct _GabbleCapsChannelManager GabbleCapsChannelManager;
-typedef struct _GabbleCapsChannelManagerIface GabbleCapsChannelManagerIface;
-
+typedef struct _GabbleCapsChannelManagerInterface GabbleCapsChannelManagerInterface;
 
 /* virtual methods */
 
@@ -91,7 +88,7 @@ void gabble_caps_channel_manager_represent_client (
     const gchar * const *cap_tokens,
     GabbleCapabilitySet *cap_set);
 
-struct _GabbleCapsChannelManagerIface {
+struct _GabbleCapsChannelManagerInterface {
     GTypeInterface parent;
 
     GabbleCapsChannelManagerResetCapsFunc reset_caps;
