@@ -83,6 +83,9 @@ struct _GabbleConnectionPresencePrivate {
 
     /* The shared status IQ handler */
     guint iq_shared_status_cb;
+
+    /* The previous presence when using shared status */
+    GabblePresenceId previous_shared_status;
 };
 
 static const TpPresenceStatusOptionalArgumentSpec gabble_status_arguments[] = {
@@ -317,6 +320,12 @@ build_shared_status_stanza (GabbleConnection *self)
   return iq;
 }
 
+static gboolean
+is_presence_away (GabblePresenceId status)
+{
+  return status == GABBLE_PRESENCE_AWAY || status == GABBLE_PRESENCE_XA;
+}
+
 static void
 set_shared_status_cb (GObject *source_object,
     GAsyncResult *res,
@@ -324,18 +333,41 @@ set_shared_status_cb (GObject *source_object,
 {
   GSimpleAsyncResult *result = G_SIMPLE_ASYNC_RESULT (user_data);
   GabbleConnection *self = GABBLE_CONNECTION (source_object);
+  GabbleConnectionPresencePrivate *priv = self->presence_priv;
+  GabblePresence *presence = self->self_presence;
   GError *error = NULL;
 
-  if (!conn_util_send_iq_finish (self, res, NULL, &error) ||
-      !conn_presence_signal_own_presence (self, NULL, &error))
+  if (!conn_util_send_iq_finish (self, res, NULL, &error))
     {
       g_simple_async_result_set_error (result,
           CONN_PRESENCE_ERROR, CONN_PRESENCE_ERROR_SET_SHARED_STATUS,
           "error setting Google shared status: %s", error->message);
     }
+  else
+    {
+      gabble_muc_factory_broadcast_presence (self->muc_factory);
 
-      g_simple_async_result_complete (result);
-      g_object_unref (result);
+      if (is_presence_away (priv->previous_shared_status))
+        {
+          /* To use away and xa we need to send a <presence/> to the server,
+           * but then GTalk also expects us to leave the status using
+           * <presence/> too. */
+          conn_presence_signal_own_presence (self, NULL, &error);
+        }
+      else if (priv->previous_shared_status == GABBLE_PRESENCE_HIDDEN &&
+          is_presence_away (presence->status))
+        {
+          /* We sent the shared status change to leave the invisibility, so
+           * now we can actually go to away / xa. */
+          conn_presence_signal_own_presence (self, NULL, &error);
+          emit_presences_changed_for_self (self);
+        }
+
+      priv->previous_shared_status = presence->status;
+    }
+
+  g_simple_async_result_complete (result);
+  g_object_unref (result);
 
   if (error != NULL)
     g_error_free (error);
@@ -349,7 +381,7 @@ insert_presence_to_shared_statuses (GabbleConnection *self)
   const gchar *show = presence->status == GABBLE_PRESENCE_DND ? "dnd" : "default";
   gchar **statuses = g_hash_table_lookup (priv->shared_statuses, show);
 
-  if (presence->status_message == NULL)
+  if (presence->status_message == NULL || is_presence_away (presence->status))
     return;
 
   if (statuses == NULL)
@@ -380,23 +412,52 @@ set_shared_status (GabbleConnection *self,
 {
   GabbleConnectionPresencePrivate *priv = self->presence_priv;
   GabblePresence *presence = self->self_presence;
-  WockyStanza *iq;
 
   g_object_ref (result);
 
-  DEBUG ("shared status invisibility is %savailable",
-      priv->shared_status_compat ? "" : "un");
+  /* Away is treated like idleness in GTalk; it's per connection and not
+   * global. To set the presence as away we use the traditional <presence/>,
+   * but, if we were invisible, we need to first leave invisibility. */
+  if (!is_presence_away (presence->status) ||
+      priv->previous_shared_status == GABBLE_PRESENCE_HIDDEN)
+    {
+      WockyStanza *iq;
 
-  if (presence->status == GABBLE_PRESENCE_HIDDEN && !priv->shared_status_compat)
-    presence->status = GABBLE_PRESENCE_DND;
+      DEBUG ("shared status invisibility is %savailable",
+          priv->shared_status_compat ? "" : "un");
 
-  insert_presence_to_shared_statuses (self);
+      if (presence->status == GABBLE_PRESENCE_HIDDEN && !priv->shared_status_compat)
+        presence->status = GABBLE_PRESENCE_DND;
 
-  iq = build_shared_status_stanza (self);
+      insert_presence_to_shared_statuses (self);
 
-  conn_util_send_iq_async (self, iq, NULL, set_shared_status_cb, result);
+      iq = build_shared_status_stanza (self);
 
-  g_object_unref (iq);
+      conn_util_send_iq_async (self, iq, NULL, set_shared_status_cb, result);
+
+      g_object_unref (iq);
+    }
+  else
+    {
+      gboolean retval;
+      GError *error = NULL;
+
+      DEBUG ("not updating shared status as it's not supported for away");
+
+      retval = conn_presence_signal_own_presence (self, NULL, &error);
+      if (!retval)
+        {
+          g_simple_async_result_set_from_error (result, error);
+          g_error_free (error);
+        }
+
+      emit_presences_changed_for_self (self);
+
+      priv->previous_shared_status = presence->status;
+
+      g_simple_async_result_complete_in_idle (result);
+      g_object_unref (result);
+    }
 }
 
 static void
@@ -803,8 +864,17 @@ store_shared_statuses (GabbleConnection *self,
         }
     }
 
+  /* - status-min-ver == 0 means that at least one resource doesn't support
+   *   Google shared status, so we fallback to "dnd".
+   * - status-min-ver == 1 means that all the resources support shared
+   *   status, but at least one doesn't support invisibility; we have to fall
+   *   fall back to "dnd".
+   * - status-miv-ver == 2 means that all the resources support shared status
+   *   with invisibility.
+   * - any other value means that the other resources will have to fall back
+   *   to version 2 for us. */
   priv->shared_status_compat =
-    g_strcmp0 (min_version, GOOGLE_SHARED_STATUS_VERSION) == 0;
+    (g_strcmp0 (min_version, "0") != 0 && g_strcmp0 (min_version, "1") != 0);
 
   if (invisible)
     {
@@ -823,12 +893,23 @@ store_shared_statuses (GabbleConnection *self,
         presence_id = GABBLE_PRESENCE_AVAILABLE;
     }
 
-  /* If we are connected, use the new shared status. If not, override with local */
-  if (base->status == TP_CONNECTION_STATUS_CONNECTED)
-    rv = gabble_presence_update (self->self_presence, resource, presence_id,
-        status_message, prio, NULL, time (NULL));
+  if (base->status != TP_CONNECTION_STATUS_CONNECTED)
+    {
+      /* Not connected, override with the local status. */
+      rv = TRUE;
+    }
+  else if (is_presence_away (self->self_presence->status))
+    {
+      /* Away presence is not overridden with remote presence because it's
+       * per connection. */
+      rv = FALSE;
+    }
   else
-    rv = TRUE;
+    {
+      /* Update with the remote presence */
+      rv = gabble_presence_update (self->self_presence, resource, presence_id,
+          status_message, prio, NULL, time (NULL));
+    }
 
   g_free (resource);
 
@@ -1003,6 +1084,10 @@ get_shared_status_async (GabbleConnection *self,
 
   conn_util_send_iq_async (self, iq, NULL, get_shared_status_cb, result);
 
+  /* We cannot use the chat status with GTalk's shared status. */
+  if (self->self_presence->status == GABBLE_PRESENCE_CHAT)
+    self->self_presence->status = GABBLE_PRESENCE_AVAILABLE;
+
   g_object_unref (iq);
 }
 
@@ -1363,6 +1448,38 @@ privacy_lists_loaded_cb (GObject *source_object,
         toggle_initial_presence_visibility_cb, user_data);
 }
 
+static void
+shared_status_toggle_initial_presence_visibility_cb (GObject *source_object,
+    GAsyncResult *result,
+    gpointer user_data)
+{
+  GabbleConnection *self = GABBLE_CONNECTION (source_object);
+  GSimpleAsyncResult *external_result = G_SIMPLE_ASYNC_RESULT (user_data);
+  GError *error = NULL;
+
+  if (!toggle_presence_visibility_finish (self, result, &error))
+    {
+      g_simple_async_result_set_from_error (external_result, error);
+      g_clear_error (&error);
+    }
+  else if (self->self_presence->status != GABBLE_PRESENCE_AWAY &&
+      self->self_presence->status != GABBLE_PRESENCE_XA)
+    {
+      /* With shared status we send the normal <presence/> only with away and
+       * extended away, but for initial status we need to send <presence/> as
+       * it also contains the caps. */
+      if (!conn_presence_signal_own_presence (self, NULL, &error))
+        {
+          g_simple_async_result_set_from_error (external_result, error);
+          g_error_free (error);
+        }
+    }
+
+  g_simple_async_result_complete_in_idle (external_result);
+
+  g_object_unref (external_result);
+}
+
 static void
 shared_status_setup_cb (GObject *source_object,
     GAsyncResult *result,
@@ -1393,7 +1510,8 @@ shared_status_setup_cb (GObject *source_object,
       g_error_free (error);
     }
 
-  get_existing_privacy_lists_async (self, privacy_lists_loaded_cb, user_data);
+  toggle_presence_visibility_async (self,
+      shared_status_toggle_initial_presence_visibility_cb, user_data);
 }
 
 void
@@ -1790,6 +1908,12 @@ status_available_cb (GObject *obj, guint status)
   TpConnectionPresenceType presence_type =
     gabble_statuses[status].presence_type;
 
+  if (base->status != TP_CONNECTION_STATUS_CONNECTED)
+    {
+      /* we just don't know yet */
+      return TRUE;
+    }
+
   /* This relies on the fact the first entries in the statuses table
    * are from gabble_base_statuses. If index to the statuses table is outside
    * the gabble_base_statuses table, the status is provided by a plugin. */
@@ -1799,36 +1923,35 @@ status_available_cb (GObject *obj, guint status)
        * lists, so any extra status should be backed by one. If it's not
        * (or if privacy lists are not supported by the server at all)
        * by the time we're connected, it's not available. */
-
-      if (base->status == TP_CONNECTION_STATUS_CONNECTED)
+      if (priv->privacy_statuses != NULL &&
+          g_hash_table_lookup (priv->privacy_statuses,
+              gabble_statuses[status].name))
         {
-          if (priv->privacy_statuses != NULL &&
-              g_hash_table_lookup (priv->privacy_statuses,
-                  gabble_statuses[status].name))
-            {
-              return TRUE;
-            }
-          else
-            {
-              return FALSE;
-            }
+          return TRUE;
         }
       else
         {
-          /* we just don't know yet */
-          return TRUE;
+          return FALSE;
         }
     }
 
-  /* If we've gone online and found that the server doesn't support invisible,
-   * reject it.
-   */
-  if (base->status == TP_CONNECTION_STATUS_CONNECTED &&
-      presence_type == TP_CONNECTION_PRESENCE_TYPE_HIDDEN &&
+  if (presence_type == TP_CONNECTION_PRESENCE_TYPE_HIDDEN &&
       priv->invisibility_method == INVISIBILITY_METHOD_NONE)
-    return FALSE;
+    {
+      /* If we've gone online and found that the server doesn't support
+       * invisible, reject it. */
+      return FALSE;
+    }
+  else if (status == GABBLE_PRESENCE_CHAT &&
+      priv->shared_statuses != NULL)
+    {
+      /* We cannot use the chat status with GTalk's shared status. */
+      return FALSE;
+    }
   else
-    return TRUE;
+    {
+      return TRUE;
+    }
 }
 
 GabblePresenceId
@@ -1863,6 +1986,7 @@ void
 conn_presence_init (GabbleConnection *conn)
 {
   conn->presence_priv = g_slice_new0 (GabbleConnectionPresencePrivate);
+  conn->presence_priv->previous_shared_status = GABBLE_PRESENCE_UNKNOWN;
 
   g_signal_connect (conn->presence_cache, "presences-updated",
       G_CALLBACK (connection_presences_updated_cb), conn);
