@@ -27,8 +27,8 @@
 #include <dbus/dbus-glib.h>
 #include <telepathy-glib/dbus.h>
 #include <telepathy-glib/errors.h>
+#include <wocky/wocky-caps-cache.h>
 
-#include "caps-cache.h"
 #include "connection.h"
 #include "debug.h"
 
@@ -67,7 +67,7 @@ gabble_connection_manager_constructed (GObject *object)
 static void
 gabble_connection_manager_finalize (GObject *object)
 {
-  gabble_caps_cache_free_shared ();
+  wocky_caps_cache_free_shared ();
   gabble_debug_free ();
 
   G_OBJECT_CLASS (gabble_connection_manager_parent_class)->finalize (object);
