@@ -52,21 +52,23 @@ G_BEGIN_DECLS
 
 /* order must match array of statuses in conn-presence.c */
 /* in increasing order of presence */
+/*< prefix=GABBLE_PRESENCE >*/
 typedef enum
 {
   GABBLE_PRESENCE_OFFLINE = 0,
   GABBLE_PRESENCE_UNKNOWN,
   GABBLE_PRESENCE_ERROR,
-  GABBLE_PRESENCE_LAST_UNAVAILABLE = GABBLE_PRESENCE_ERROR,
+  GABBLE_PRESENCE_LAST_UNAVAILABLE = GABBLE_PRESENCE_ERROR, /*< skip >*/
   GABBLE_PRESENCE_HIDDEN,
   GABBLE_PRESENCE_XA,
   GABBLE_PRESENCE_AWAY,
   GABBLE_PRESENCE_DND,
   GABBLE_PRESENCE_AVAILABLE,
   GABBLE_PRESENCE_CHAT,
-  NUM_GABBLE_PRESENCES
+  NUM_GABBLE_PRESENCES /*< skip >*/
 } GabblePresenceId;
 
+/*< flags >*/
 typedef enum
 {
   GABBLE_CONNECTION_FEATURES_NONE = 0,
@@ -216,6 +218,7 @@ void _gabble_connection_acknowledge_set_iq (GabbleConnection *conn,
     LmMessage *iq);
 void _gabble_connection_send_iq_error (GabbleConnection *conn,
     LmMessage *message, GabbleXmppError error, const gchar *errmsg);
+void gabble_connection_update_last_use (GabbleConnection *conn);
 
 const char *_gabble_connection_find_conference_server (GabbleConnection *);
 gchar *gabble_connection_get_canonical_room_name (GabbleConnection *conn,
