@@ -25,6 +25,8 @@
 # include <unistd.h>
 #endif
 
+#include <glib/gstdio.h>
+
 #include <telepathy-glib/debug.h>
 #include <telepathy-glib/debug-sender.h>
 #include <telepathy-glib/run.h>
@@ -77,18 +79,17 @@ log_handler (const gchar *log_domain,
       if (stamp_logs)
         {
           GTimeVal now;
-          gchar now_str[32];
+          gchar *now_str = NULL;
           gchar *tmp;
-          struct tm tm;
 
           g_get_current_time (&now);
-          localtime_r (&(now.tv_sec), &tm);
-          strftime (now_str, 32, "%Y-%m-%d %H:%M:%S", &tm);
+          now_str = g_time_val_to_iso8601 (&now);
           tmp = g_strdup_printf ("%s.%06ld: %s",
             now_str, now.tv_usec, message);
 
           g_log_default_handler (log_domain, log_level, tmp, NULL);
 
+          g_free (now_str);
           g_free (tmp);
         }
       else
@@ -117,15 +118,39 @@ gabble_init (void)
   wocky_init ();
 }
 
+static void
+try_to_delete_old_caps_cache (void)
+{
+  gchar *cache = g_build_path (G_DIR_SEPARATOR_S,
+      g_get_user_cache_dir (), "telepathy", "gabble", "caps-cache.db",
+      NULL);
+
+  if (g_file_test (cache, G_FILE_TEST_IS_REGULAR))
+    {
+      /* fire and forget */
+      g_unlink (cache);
+    }
+
+  g_free (cache);
+}
+
 int
 gabble_main (int argc,
     char **argv)
 {
   GabblePluginLoader *loader;
   int out;
+  GLogLevelFlags fatal_mask;
 
   tp_debug_divert_messages (g_getenv ("GABBLE_LOGFILE"));
 
+#ifdef ENABLE_FATAL_CRITICALS
+  /* make critical warnings fatal */
+  fatal_mask = g_log_set_always_fatal (G_LOG_FATAL_MASK);
+  fatal_mask |= G_LOG_LEVEL_CRITICAL;
+  g_log_set_always_fatal (fatal_mask);
+#endif
+
 #ifdef ENABLE_DEBUG
   gabble_debug_set_flags_from_env ();
 
@@ -147,6 +172,8 @@ gabble_main (int argc,
 
   loader = gabble_plugin_loader_dup ();
 
+  try_to_delete_old_caps_cache ();
+
   out = tp_run_connection_manager ("telepathy-gabble", VERSION,
       construct_cm, argc, argv);
 
