@@ -393,17 +393,10 @@ new_im_channel (GabbleImFactory *fac,
 static void
 gabble_im_factory_close_all (GabbleImFactory *self)
 {
-  /* Use a temporary variable because we don't want
+  /* Use a temporary variable (the macro does this) because we don't want
    * im_channel_closed_cb to remove the channel from the hash table a
    * second time */
-  if (self->priv->channels != NULL)
-    {
-      GHashTable *tmp = self->priv->channels;
-
-      DEBUG ("closing channels");
-      self->priv->channels = NULL;
-      g_hash_table_destroy (tmp);
-    }
+  tp_clear_pointer (&self->priv->channels, g_hash_table_destroy);
 
   if (self->priv->status_changed_id != 0)
     {
@@ -417,9 +410,9 @@ gabble_im_factory_close_all (GabbleImFactory *self)
       DEBUG ("removing callbacks");
       lm_connection_unregister_message_handler (self->priv->conn->lmconn,
           self->priv->message_cb, LM_MESSAGE_TYPE_MESSAGE);
-      lm_message_handler_unref (self->priv->message_cb);
-      self->priv->message_cb = NULL;
     }
+
+  tp_clear_pointer (&self->priv->message_cb, lm_message_handler_unref);
 }
 
 
@@ -457,6 +450,12 @@ gabble_im_factory_get_contact_caps (GabbleCapsChannelManager *manager,
   /* We don't need to check this contact's capabilities, we assume every
    * contact support text channels. */
 
+  /* NOTE: if any more contact caps are added here which are not
+   * assumed then be sure to change the implementation of
+   * gabble_connection_get_handle_contact_capabilities, as that uses
+   * this function to fill the contact caps hash for all contacts
+   * (even offline). */
+
   GValue monster = {0, };
   GHashTable *fixed_properties;
   GValue *channel_type_value;
@@ -676,7 +675,7 @@ static void
 caps_channel_manager_iface_init (gpointer g_iface,
                                  gpointer iface_data)
 {
-  GabbleCapsChannelManagerIface *iface = g_iface;
+  GabbleCapsChannelManagerInterface *iface = g_iface;
 
   iface->get_contact_caps = gabble_im_factory_get_contact_caps;
 }
