@@ -25,12 +25,13 @@
 #include <string.h>
 #include <glib.h>
 
-#include <loudmouth/loudmouth.h>
+#include <wocky/wocky-c2s-porter.h>
 #include <libsoup/soup.h>
 
 #define DEBUG_FLAG GABBLE_DEBUG_MEDIA
 
 #include "connection.h"
+#include "conn-util.h"
 #include "debug.h"
 #include "gabble-signals-marshal.h"
 #include "jingle-share.h"
@@ -64,8 +65,8 @@ enum
 struct _GabbleJingleFactoryPrivate
 {
   GabbleConnection *conn;
-  LmMessageHandler *jingle_cb;
-  LmMessageHandler *jingle_info_cb;
+  guint jingle_handler_id;
+  guint jingle_info_handler_id;
   GHashTable *content_types;
   GHashTable *transports;
 
@@ -88,8 +89,10 @@ struct _GabbleJingleFactoryPrivate
   gboolean dispose_has_run;
 };
 
-static LmHandlerResult jingle_cb (LmMessageHandler *handler,
-    LmConnection *lmconn, LmMessage *message, gpointer user_data);
+static gboolean jingle_cb (
+    WockyPorter *porter,
+    WockyStanza *msg,
+    gpointer user_data);
 static GabbleJingleSession *create_session (GabbleJingleFactory *fac,
     const gchar *sid,
     TpHandle peer,
@@ -104,6 +107,10 @@ static void session_terminated_cb (GabbleJingleSession *sess,
 
 static void connection_status_changed_cb (GabbleConnection *conn,
     guint status, guint reason, GabbleJingleFactory *self);
+static void connection_porter_available_cb (
+    GabbleConnection *conn,
+    WockyPorter *porter,
+    gpointer user_data);
 
 #define RELAY_HTTP_TIMEOUT 5
 
@@ -132,8 +139,6 @@ gabble_jingle_factory_init (GabbleJingleFactory *obj)
   priv->content_types = g_hash_table_new_full (g_str_hash, g_str_equal,
       NULL, NULL);
 
-  priv->jingle_cb = NULL;
-
   priv->conn = NULL;
   priv->dispose_has_run = FALSE;
   priv->relay_http_port = 80;
@@ -252,70 +257,27 @@ take_stun_server (GabbleJingleFactory *self,
 }
 
 
-static LmHandlerResult
-got_jingle_info_stanza (GabbleJingleFactory *fac,
-    LmMessage *message)
+static void
+got_jingle_info_stanza (
+    GabbleJingleFactory *fac,
+    WockyStanza *stanza)
 {
-  GabbleJingleFactoryPrivate *priv = fac->priv;
-  LmMessageSubType sub_type;
-  LmMessageNode *query_node, *node;
-  const gchar *from = wocky_stanza_get_from (message);
+  WockyNode *node, *query_node;
 
-  if (from != NULL)
-    {
-      TpBaseConnection *base_conn = TP_BASE_CONNECTION (priv->conn);
-      TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
-          base_conn, TP_HANDLE_TYPE_CONTACT);
-      TpHandle sender = tp_handle_lookup (contact_repo, from, NULL, NULL);
-
-      if (sender != base_conn->self_handle)
-        {
-          DEBUG ("ignoring jingleinfo from '%s', not ourself nor the server",
-              from);
-          return LM_HANDLER_RESULT_ALLOW_MORE_HANDLERS;
-        }
-    }
-
-  query_node = lm_message_node_get_child_with_namespace (
-      wocky_stanza_get_top_node (message), "query", NS_GOOGLE_JINGLE_INFO);
+  query_node = wocky_node_get_child_ns (
+      wocky_stanza_get_top_node (stanza), "query", NS_GOOGLE_JINGLE_INFO);
 
   if (query_node == NULL)
-    return LM_HANDLER_RESULT_ALLOW_MORE_HANDLERS;
-
-  sub_type = lm_message_get_sub_type (message);
-
-  if (sub_type == LM_MESSAGE_SUB_TYPE_ERROR)
-    {
-      GabbleXmppError xmpp_error = XMPP_ERROR_UNDEFINED_CONDITION;
-
-      node = lm_message_node_get_child (wocky_stanza_get_top_node (message),
-          "error");
-      if (node != NULL)
-        {
-          xmpp_error = gabble_xmpp_error_from_node (node, NULL);
-        }
-
-      DEBUG ("jingle info error: %s", gabble_xmpp_error_string (xmpp_error));
-
-      return LM_HANDLER_RESULT_REMOVE_MESSAGE;
-    }
-
-  if (sub_type != LM_MESSAGE_SUB_TYPE_RESULT &&
-      sub_type != LM_MESSAGE_SUB_TYPE_SET)
-    {
-      DEBUG ("jingle info: unexpected IQ type, ignoring");
-
-      return LM_HANDLER_RESULT_ALLOW_MORE_HANDLERS;
-    }
+    return;
 
   if (fac->priv->get_stun_from_jingle)
-    node = lm_message_node_get_child (query_node, "stun");
+    node = wocky_node_get_child (query_node, "stun");
   else
     node = NULL;
 
   if (node != NULL)
     {
-      node = lm_message_node_get_child (node, "server");
+      node = wocky_node_get_child (node, "server");
 
       if (node != NULL)
         {
@@ -323,8 +285,8 @@ got_jingle_info_stanza (GabbleJingleFactory *fac,
           const gchar *port_attr;
           guint port = GABBLE_PARAMS_DEFAULT_STUN_PORT;
 
-          server = lm_message_node_get_attribute (node, "host");
-          port_attr = lm_message_node_get_attribute (node, "udp");
+          server = wocky_node_get_attribute (node, "host");
+          port_attr = wocky_node_get_attribute (node, "udp");
 
           if (port_attr != NULL)
             port = atoi (port_attr);
@@ -338,19 +300,16 @@ got_jingle_info_stanza (GabbleJingleFactory *fac,
         }
     }
 
-  node = lm_message_node_get_child (query_node, "relay");
+  node = wocky_node_get_child (query_node, "relay");
 
   if (node != NULL)
     {
-      LmMessageNode *subnode;
-
-      subnode = lm_message_node_get_child (node, "token");
+      WockyNode *subnode = wocky_node_get_child (node, "token");
 
       if (subnode != NULL)
         {
-          const gchar *token;
+          const gchar *token = subnode->content;
 
-          token = lm_message_node_get_value (subnode);
           if (token != NULL)
             {
               DEBUG ("jingle info: got Google relay token %s", token);
@@ -359,14 +318,14 @@ got_jingle_info_stanza (GabbleJingleFactory *fac,
             }
         }
 
-      subnode = lm_message_node_get_child (node, "server");
+      subnode = wocky_node_get_child (node, "server");
 
       if (subnode != NULL)
         {
           const gchar *server;
           const gchar *port;
 
-          server = lm_message_node_get_attribute (subnode, "host");
+          server = wocky_node_get_attribute (subnode, "host");
 
           if (server != NULL)
             {
@@ -379,7 +338,7 @@ got_jingle_info_stanza (GabbleJingleFactory *fac,
             {
               /* this is not part of the real protocol, but we can't listen on
                * port 80 in an unprivileged regression test */
-              port = lm_message_node_get_attribute (subnode,
+              port = wocky_node_get_attribute (subnode,
                   "gabble-test-http-port");
 
               if (port != NULL)
@@ -393,7 +352,7 @@ got_jingle_info_stanza (GabbleJingleFactory *fac,
           /* FIXME: these are not really actually used anywhere at
            * the moment, because we get the same info when creating
            * relay session. */
-          port = lm_message_node_get_attribute (subnode, "udp");
+          port = wocky_node_get_attribute (subnode, "udp");
 
           if (port != NULL)
             {
@@ -401,7 +360,7 @@ got_jingle_info_stanza (GabbleJingleFactory *fac,
               fac->priv->relay_udp = atoi (port);
             }
 
-          port = lm_message_node_get_attribute (subnode, "tcp");
+          port = wocky_node_get_attribute (subnode, "tcp");
 
           if (port != NULL)
             {
@@ -409,7 +368,7 @@ got_jingle_info_stanza (GabbleJingleFactory *fac,
               fac->priv->relay_tcp = atoi (port);
             }
 
-          port = lm_message_node_get_attribute (subnode, "tcpssl");
+          port = wocky_node_get_attribute (subnode, "tcpssl");
 
           if (port != NULL)
             {
@@ -420,75 +379,60 @@ got_jingle_info_stanza (GabbleJingleFactory *fac,
         }
 
     }
-
-  if (sub_type == LM_MESSAGE_SUB_TYPE_SET)
-    {
-      _gabble_connection_acknowledge_set_iq (priv->conn, message);
-    }
-
-  return LM_HANDLER_RESULT_REMOVE_MESSAGE;
 }
 
-/*
- * jingle_info_cb
- *
- * Called by loudmouth when we get an incoming <iq>. This handler
- * is concerned only with Jingle info queries.
- */
-static LmHandlerResult
-jingle_info_cb (LmMessageHandler *handler,
-                LmConnection *lmconn,
-                LmMessage *message,
-                gpointer user_data)
+static gboolean
+jingle_info_cb (
+    WockyPorter *porter,
+    WockyStanza *stanza,
+    gpointer user_data)
 {
   GabbleJingleFactory *fac = GABBLE_JINGLE_FACTORY (user_data);
 
-  return got_jingle_info_stanza (fac, message);
+  got_jingle_info_stanza (fac, stanza);
+  wocky_porter_acknowledge_iq (porter, stanza, NULL);
+
+  return TRUE;
 }
 
-static LmHandlerResult
-jingle_info_reply_cb (GabbleConnection *conn,
-    LmMessage *sent_msg,
-    LmMessage *reply_msg,
-    GObject *factory_obj,
+static void
+jingle_info_reply_cb (
+    GObject *source,
+    GAsyncResult *result,
     gpointer user_data)
 {
-  GabbleJingleFactory *fac = GABBLE_JINGLE_FACTORY (factory_obj);
+  GabbleJingleFactory *fac = GABBLE_JINGLE_FACTORY (user_data);
+  WockyStanza *reply = NULL;
+  GError *error = NULL;
 
-  return got_jingle_info_stanza (fac, reply_msg);
+  if (conn_util_send_iq_finish (GABBLE_CONNECTION (source), result, &reply,
+          &error))
+    {
+      got_jingle_info_stanza (fac, reply);
+    }
+  else
+    {
+      DEBUG ("jingle info request failed: %s", error->message);
+      g_clear_error (&error);
+    }
+
+  tp_clear_object (&reply);
 }
 
 static void
 jingle_info_send_request (GabbleJingleFactory *fac)
 {
   GabbleJingleFactoryPrivate *priv = fac->priv;
-  TpBaseConnection *base = (TpBaseConnection *) priv->conn;
-  LmMessage *msg;
-  LmMessageNode *node;
-  const gchar *jid;
-  GError *error = NULL;
-  TpHandleRepoIface *contact_handles = tp_base_connection_get_handles (base,
-      TP_HANDLE_TYPE_CONTACT);
+  const gchar *jid = conn_util_get_bare_self_jid (priv->conn);
+  WockyStanza *stanza = wocky_stanza_build (
+      WOCKY_STANZA_TYPE_IQ, WOCKY_STANZA_SUB_TYPE_GET, NULL, jid,
+      '(', "query", ':', NS_GOOGLE_JINGLE_INFO, ')', NULL);
 
-  jid = tp_handle_inspect (contact_handles, base->self_handle);
-  msg = lm_message_new_with_sub_type (jid, LM_MESSAGE_TYPE_IQ,
-      LM_MESSAGE_SUB_TYPE_GET);
+  conn_util_send_iq_async (priv->conn, stanza, NULL, jingle_info_reply_cb, fac);
 
-  node = lm_message_node_add_child (
-      wocky_stanza_get_top_node (msg), "query", NULL);
-  lm_message_node_set_attribute (node, "xmlns", NS_GOOGLE_JINGLE_INFO);
-
-  if (!_gabble_connection_send_with_reply (priv->conn, msg,
-        jingle_info_reply_cb, G_OBJECT (fac), fac, &error))
-    {
-      DEBUG ("jingle info send failed: %s\n", error->message);
-      g_error_free (error);
-    }
-
-  lm_message_unref (msg);
+  g_object_unref (stanza);
 }
 
-
 static void
 gabble_jingle_factory_dispose (GObject *object)
 {
@@ -552,32 +496,26 @@ gabble_jingle_factory_set_property (GObject *object,
   }
 }
 
-static GObject *
-gabble_jingle_factory_constructor (GType type,
-                                   guint n_props,
-                                   GObjectConstructParam *props)
+static void
+gabble_jingle_factory_constructed (GObject *obj)
 {
-  GObject *obj;
-  GabbleJingleFactory *self;
-  GabbleJingleFactoryPrivate *priv;
+  GabbleJingleFactory *self = GABBLE_JINGLE_FACTORY (obj);
+  GabbleJingleFactoryPrivate *priv = self->priv;
+  GObjectClass *parent = G_OBJECT_CLASS (gabble_jingle_factory_parent_class);
 
-  obj = G_OBJECT_CLASS (gabble_jingle_factory_parent_class)->
-      constructor (type, n_props, props);
+  if (parent->constructed != NULL)
+    parent->constructed (obj);
 
-  self = GABBLE_JINGLE_FACTORY (obj);
-  priv = self->priv;
-
-  /* FIXME: why was this in _constructed in media factory? */
   gabble_signal_connect_weak (priv->conn, "status-changed",
       (GCallback) connection_status_changed_cb, G_OBJECT (self));
+  gabble_signal_connect_weak (priv->conn, "porter-available",
+      (GCallback) connection_porter_available_cb, G_OBJECT (self));
 
   jingle_share_register (self);
   jingle_media_rtp_register (self);
   jingle_transport_google_register (self);
   jingle_transport_rawudp_register (self);
   jingle_transport_iceudp_register (self);
-
-  return obj;
 }
 
 static void
@@ -588,7 +526,7 @@ gabble_jingle_factory_class_init (GabbleJingleFactoryClass *cls)
 
   g_type_class_add_private (cls, sizeof (GabbleJingleFactoryPrivate));
 
-  object_class->constructor = gabble_jingle_factory_constructor;
+  object_class->constructed = gabble_jingle_factory_constructed;
   object_class->get_property = gabble_jingle_factory_get_property;
   object_class->set_property = gabble_jingle_factory_set_property;
   object_class->dispose = gabble_jingle_factory_dispose;
@@ -625,22 +563,6 @@ connection_status_changed_cb (GabbleConnection *conn,
     {
     case TP_CONNECTION_STATUS_CONNECTING:
       g_assert (priv->conn != NULL);
-
-      g_assert (priv->jingle_cb == NULL);
-      g_assert (priv->jingle_info_cb == NULL);
-
-      priv->jingle_cb = lm_message_handler_new (jingle_cb,
-          self, NULL);
-      lm_connection_register_message_handler (priv->conn->lmconn,
-          priv->jingle_cb, LM_MESSAGE_TYPE_IQ,
-          LM_HANDLER_PRIORITY_NORMAL);
-
-      priv->jingle_info_cb = lm_message_handler_new (
-          jingle_info_cb, self, NULL);
-      lm_connection_register_message_handler (priv->conn->lmconn,
-          priv->jingle_info_cb, LM_MESSAGE_TYPE_IQ,
-          LM_HANDLER_PRIORITY_NORMAL);
-
       break;
 
     case TP_CONNECTION_STATUS_CONNECTED:
@@ -681,20 +603,44 @@ connection_status_changed_cb (GabbleConnection *conn,
       break;
 
     case TP_CONNECTION_STATUS_DISCONNECTED:
-      if (priv->jingle_cb != NULL)
+      if (priv->jingle_handler_id != 0)
         {
-          lm_connection_unregister_message_handler (priv->conn->lmconn,
-              priv->jingle_cb, LM_MESSAGE_TYPE_IQ);
-          lm_connection_unregister_message_handler (priv->conn->lmconn,
-              priv->jingle_info_cb, LM_MESSAGE_TYPE_IQ);
+          WockyPorter *p = wocky_session_get_porter (priv->conn->session);
+
+          wocky_porter_unregister_handler (p, priv->jingle_handler_id);
+          wocky_porter_unregister_handler (p, priv->jingle_info_handler_id);
+          priv->jingle_handler_id = 0;
+          priv->jingle_info_handler_id = 0;
         }
 
-      tp_clear_pointer (&priv->jingle_cb, lm_message_handler_unref);
-      tp_clear_pointer (&priv->jingle_info_cb, lm_message_handler_unref);
       break;
     }
 }
 
+static void
+connection_porter_available_cb (
+    GabbleConnection *conn,
+    WockyPorter *porter,
+    gpointer user_data)
+{
+  GabbleJingleFactory *self = GABBLE_JINGLE_FACTORY (user_data);
+  GabbleJingleFactoryPrivate *priv = self->priv;
+
+  g_assert (priv->jingle_handler_id == 0);
+
+  /* TODO: we could match different dialects here maybe? */
+  priv->jingle_handler_id = wocky_porter_register_handler_from_anyone (porter,
+      WOCKY_STANZA_TYPE_IQ, WOCKY_STANZA_SUB_TYPE_SET,
+      WOCKY_PORTER_HANDLER_PRIORITY_NORMAL, jingle_cb, self,
+      NULL);
+
+  priv->jingle_info_handler_id = wocky_c2s_porter_register_handler_from_server (
+      WOCKY_C2S_PORTER (porter),
+      WOCKY_STANZA_TYPE_IQ, WOCKY_STANZA_SUB_TYPE_SET,
+      WOCKY_PORTER_HANDLER_PRIORITY_NORMAL, jingle_info_cb, self,
+      '(', "query", ':', NS_GOOGLE_JINGLE_INFO, ')', NULL);
+}
+
 /* The 'session' map is keyed by:
  * "<peer's handle>\n<peer's jid>\n<session id>"
  */
@@ -786,11 +732,11 @@ ensure_session (GabbleJingleFactory *self,
   return sess;
 }
 
-static LmHandlerResult
-jingle_cb (LmMessageHandler *handler,
-           LmConnection *lmconn,
-           LmMessage *msg,
-           gpointer user_data)
+static gboolean
+jingle_cb (
+    WockyPorter *porter,
+    WockyStanza *msg,
+    gpointer user_data)
 {
   GabbleJingleFactory *self = GABBLE_JINGLE_FACTORY (user_data);
   GabbleJingleFactoryPrivate *priv = self->priv;
@@ -803,10 +749,10 @@ jingle_cb (LmMessageHandler *handler,
 
   /* see if it's a jingle message and detect dialect */
   sid = gabble_jingle_session_detect (msg, &action, &dialect);
-  from = lm_message_node_get_attribute (lm_message_get_node (msg), "from");
+  from = wocky_stanza_get_from (msg);
 
   if (sid == NULL || from == NULL)
-    return LM_HANDLER_RESULT_ALLOW_MORE_HANDLERS;
+    return FALSE;
 
   sess = ensure_session (self, sid, from, action, dialect, &new_session,
       &error);
@@ -824,7 +770,7 @@ jingle_cb (LmMessageHandler *handler,
   /* all went well, we can acknowledge the IQ */
   _gabble_connection_acknowledge_set_iq (priv->conn, msg);
 
-  return LM_HANDLER_RESULT_REMOVE_MESSAGE;
+  return TRUE;
 
 REQUEST_ERROR:
   g_assert (error != NULL);
@@ -838,7 +784,7 @@ REQUEST_ERROR:
   if (sess != NULL && new_session)
     gabble_jingle_session_terminate (sess, JINGLE_REASON_UNKNOWN, NULL, NULL);
 
-  return LM_HANDLER_RESULT_REMOVE_MESSAGE;
+  return TRUE;
 }
 
 /*
