@@ -49,7 +49,6 @@
 #include "util.h"
 
 static void stream_handler_iface_init (gpointer, gpointer);
-static void dbus_properties_iface_init (gpointer, gpointer);
 
 G_DEFINE_TYPE_WITH_CODE(GabbleMediaStream,
     gabble_media_stream,
@@ -57,7 +56,7 @@ G_DEFINE_TYPE_WITH_CODE(GabbleMediaStream,
     G_IMPLEMENT_INTERFACE (TP_TYPE_SVC_MEDIA_STREAM_HANDLER,
       stream_handler_iface_init);
     G_IMPLEMENT_INTERFACE (TP_TYPE_SVC_DBUS_PROPERTIES,
-      dbus_properties_iface_init)
+      tp_dbus_properties_mixin_iface_init);
     )
 
 /* signal enum */
@@ -102,14 +101,19 @@ struct _GabbleMediaStreamPrivate
   guint id;
   guint media_type;
 
-  GValue native_codecs;     /* intersected codec list */
+  gboolean local_codecs_set;
 
   /* Whether we're waiting for a codec intersection from the streaming
    * implementation. If FALSE, SupportedCodecs is a no-op.
    */
   gboolean awaiting_intersection;
 
+  GValue local_rtp_hdrexts;
+  GValue local_feedback_messages;
+
   GValue remote_codecs;
+  GValue remote_rtp_hdrexts;
+  GValue remote_feedback_messages;
   GValue remote_candidates;
 
   guint remote_candidate_count;
@@ -134,15 +138,15 @@ struct _GabbleMediaStreamPrivate
   unsigned created_locally:1;
 };
 
-static void push_remote_codecs (GabbleMediaStream *stream);
+static void push_remote_media_description (GabbleMediaStream *stream);
 static void push_remote_candidates (GabbleMediaStream *stream);
 static void push_playing (GabbleMediaStream *stream);
 static void push_sending (GabbleMediaStream *stream);
 
 static void new_remote_candidates_cb (GabbleJingleContent *content,
     GList *clist, GabbleMediaStream *stream);
-static void new_remote_codecs_cb (GabbleJingleContent *content,
-    GList *clist, GabbleMediaStream *stream);
+static void new_remote_media_description_cb (GabbleJingleContent *content,
+    JingleMediaDescription *md, GabbleMediaStream *stream);
 static void content_state_changed_cb (GabbleJingleContent *c,
      GParamSpec *pspec, GabbleMediaStream *stream);
 static void content_senders_changed_cb (GabbleJingleContent *c,
@@ -205,17 +209,26 @@ gabble_media_stream_init (GabbleMediaStream *self)
       TP_ARRAY_TYPE_MEDIA_STREAM_HANDLER_CANDIDATE_LIST;
   GType codec_list_type =
       TP_ARRAY_TYPE_MEDIA_STREAM_HANDLER_CODEC_LIST;
+  GType rtp_hdrext_list_type = TP_ARRAY_TYPE_RTP_HEADER_EXTENSIONS_LIST;
+  GType fb_msg_map_type  = TP_HASH_TYPE_RTCP_FEEDBACK_MESSAGE_MAP;
 
   self->priv = priv;
 
-  g_value_init (&priv->native_codecs, codec_list_type);
-  g_value_take_boxed (&priv->native_codecs,
-      dbus_g_type_specialized_construct (codec_list_type));
+  g_value_init (&priv->local_rtp_hdrexts, rtp_hdrext_list_type);
+  g_value_init (&priv->local_feedback_messages, fb_msg_map_type);
 
   g_value_init (&priv->remote_codecs, codec_list_type);
   g_value_take_boxed (&priv->remote_codecs,
       dbus_g_type_specialized_construct (codec_list_type));
 
+  g_value_init (&priv->remote_rtp_hdrexts, rtp_hdrext_list_type);
+  g_value_take_boxed (&priv->remote_rtp_hdrexts,
+      dbus_g_type_specialized_construct (rtp_hdrext_list_type));
+
+  g_value_init (&priv->remote_feedback_messages, fb_msg_map_type);
+  g_value_take_boxed (&priv->remote_feedback_messages,
+      dbus_g_type_specialized_construct (fb_msg_map_type));
+
   g_value_init (&priv->remote_candidates, candidate_list_type);
   g_value_take_boxed (&priv->remote_candidates,
       dbus_g_type_specialized_construct (candidate_list_type));
@@ -228,13 +241,15 @@ _get_initial_codecs_and_candidates (gpointer user_data)
 {
   GabbleMediaStream *stream = GABBLE_MEDIA_STREAM (user_data);
   GabbleMediaStreamPrivate *priv = stream->priv;
+  JingleMediaDescription *md;
 
   priv->initial_getter_id = 0;
 
   /* we can immediately get the codecs if we're responder */
-  new_remote_codecs_cb (priv->content,
-      gabble_jingle_media_rtp_get_remote_codecs (GABBLE_JINGLE_MEDIA_RTP (priv->content)),
-      stream);
+  md = gabble_jingle_media_rtp_get_remote_media_description (
+      GABBLE_JINGLE_MEDIA_RTP (priv->content));
+  if (md != NULL)
+    new_remote_media_description_cb (priv->content, md, stream);
 
   /* if any candidates arrived before idle loop had the chance to excute
    * us (e.g. specified in session-initiate/content-add), we don't want to
@@ -252,7 +267,7 @@ gabble_media_stream_constructor (GType type, guint n_props,
   GObject *obj;
   GabbleMediaStream *stream;
   GabbleMediaStreamPrivate *priv;
-  DBusGConnection *bus;
+  TpDBusDaemon *bus;
   GabbleConnection *connection;
   gchar *stun_server;
   guint stun_port;
@@ -287,11 +302,11 @@ gabble_media_stream_constructor (GType type, guint n_props,
       g_ptr_array_add (priv->stun_servers, va);
     }
 
-  g_object_unref (connection);
-
   /* go for the bus */
-  bus = tp_get_bus ();
-  dbus_g_connection_register_g_object (bus, priv->object_path, obj);
+  bus = tp_base_connection_get_dbus_daemon ((TpBaseConnection *) connection);
+  tp_dbus_daemon_register_object (bus, priv->object_path, obj);
+
+  g_object_unref (connection);
 
   update_direction (stream, priv->content);
 
@@ -447,8 +462,8 @@ gabble_media_stream_set_property (GObject      *object,
 
       /* we need this also, if we're the initiator of the stream
        * (so remote codecs arrive later) */
-      gabble_signal_connect_weak (priv->content, "remote-codecs",
-          (GCallback) new_remote_codecs_cb, object);
+      gabble_signal_connect_weak (priv->content, "remote-media-description",
+          (GCallback) new_remote_media_description_cb, object);
 
       gabble_signal_connect_weak (priv->content, "notify::state",
           (GCallback) content_state_changed_cb, object);
@@ -487,6 +502,21 @@ gabble_media_stream_class_init (GabbleMediaStreamClass *gabble_media_stream_clas
 {
   GObjectClass *object_class = G_OBJECT_CLASS (gabble_media_stream_class);
   GParamSpec *param_spec;
+  static TpDBusPropertiesMixinPropImpl stream_handler_props[] = {
+      { "RelayInfo", "relay-info", NULL },
+      { "STUNServers", "stun-servers", NULL },
+      { "NATTraversal", "nat-traversal", NULL },
+      { "CreatedLocally", "created-locally", NULL },
+      { NULL }
+  };
+  static TpDBusPropertiesMixinIfaceImpl prop_interfaces[] = {
+      { TP_IFACE_MEDIA_STREAM_HANDLER,
+        tp_dbus_properties_mixin_getter_gobject_properties,
+        NULL,
+        stream_handler_props,
+      },
+      { NULL }
+  };
 
   g_type_class_add_private (gabble_media_stream_class,
       sizeof (GabbleMediaStreamPrivate));
@@ -635,6 +665,10 @@ gabble_media_stream_class_init (GabbleMediaStreamClass *gabble_media_stream_clas
       G_OBJECT_CLASS_TYPE (gabble_media_stream_class),
       G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED, 0, NULL, NULL,
       g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
+
+  gabble_media_stream_class->props_class.interfaces = prop_interfaces;
+  tp_dbus_properties_mixin_class_init (object_class,
+      G_STRUCT_OFFSET (GabbleMediaStreamClass, props_class));
 }
 
 void
@@ -658,11 +692,8 @@ gabble_media_stream_dispose (GObject *object)
 
   priv->dispose_has_run = TRUE;
 
-  g_object_unref (priv->content);
-  priv->content = NULL;
-
-  g_free (self->name);
-  self->name = NULL;
+  tp_clear_object (&priv->content);
+  tp_clear_pointer (&self->name, g_free);
 
   if (G_OBJECT_CLASS (gabble_media_stream_parent_class)->dispose)
     G_OBJECT_CLASS (gabble_media_stream_parent_class)->dispose (object);
@@ -684,9 +715,12 @@ gabble_media_stream_finalize (GObject *object)
   if (priv->relay_info != NULL)
     g_boxed_free (TP_ARRAY_TYPE_STRING_VARIANT_MAP_LIST, priv->relay_info);
 
-  g_value_unset (&priv->native_codecs);
+  g_value_unset (&priv->local_rtp_hdrexts);
+  g_value_unset (&priv->local_feedback_messages);
 
   g_value_unset (&priv->remote_codecs);
+  g_value_unset (&priv->remote_rtp_hdrexts);
+  g_value_unset (&priv->remote_feedback_messages);
   g_value_unset (&priv->remote_candidates);
 
   G_OBJECT_CLASS (gabble_media_stream_parent_class)->finalize (object);
@@ -703,13 +737,6 @@ gabble_media_stream_codec_choice (TpSvcMediaStreamHandler *iface,
                                   guint codec_id,
                                   DBusGMethodInvocation *context)
 {
-  GabbleMediaStream *self = GABBLE_MEDIA_STREAM (iface);
-  GabbleMediaStreamPrivate *priv;
-
-  g_assert (GABBLE_IS_MEDIA_STREAM (self));
-
-  priv = self->priv;
-
   tp_svc_media_stream_handler_return_from_codec_choice (context);
 }
 
@@ -720,15 +747,10 @@ gabble_media_stream_error (GabbleMediaStream *self,
                            const gchar *message,
                            GError **error)
 {
-  GabbleMediaStreamPrivate *priv;
-
   g_assert (GABBLE_IS_MEDIA_STREAM (self));
 
-  priv = self->priv;
-
   DEBUG ( "Media.StreamHandler::Error called, error %u (%s) -- emitting signal",
       errno, message);
-
   g_signal_emit (self, signals[ERROR], 0, errno, message);
 
   return TRUE;
@@ -836,13 +858,6 @@ static void
 gabble_media_stream_native_candidates_prepared (TpSvcMediaStreamHandler *iface,
                                                 DBusGMethodInvocation *context)
 {
-  GabbleMediaStream *self = GABBLE_MEDIA_STREAM (iface);
-  GabbleMediaStreamPrivate *priv;
-
-  g_assert (GABBLE_IS_MEDIA_STREAM (self));
-
-  priv = self->priv;
-
   tp_svc_media_stream_handler_return_from_native_candidates_prepared (context);
 }
 
@@ -940,7 +955,7 @@ gabble_media_stream_new_native_candidate (TpSvcMediaStreamHandler *iface,
           /* generation */
           0,
           /* preference */
-          g_value_get_double (g_value_array_get_nth (transport, 6)),
+          (int) (g_value_get_double (g_value_array_get_nth (transport, 6)) * 65536),
           /* username */
           g_value_get_string (g_value_array_get_nth (transport, 8)),
           /* password */
@@ -984,7 +999,7 @@ gabble_media_stream_ready (TpSvcMediaStreamHandler *iface,
     {
       g_object_set (self, "ready", TRUE, NULL);
 
-      push_remote_codecs (self);
+      push_remote_media_description (self);
       push_remote_candidates (self);
       push_playing (self);
       push_sending (self);
@@ -1012,14 +1027,17 @@ pass_local_codecs (GabbleMediaStream *stream,
                    GError **error)
 {
   GabbleMediaStreamPrivate *priv = stream->priv;
-  GList *li = NULL;
-  JingleCodec *c;
   guint i;
+  JingleMediaDescription *md;
+  const GPtrArray *hdrexts;
+  GHashTable *fbs;
 
   DEBUG ("putting list of %d supported codecs from stream-engine into cache",
       codecs->len);
 
-  g_value_set_boxed (&priv->native_codecs, codecs);
+  md = jingle_media_description_new ();
+
+  fbs = g_value_get_boxed (&priv->local_feedback_messages);
 
   for (i = 0; i < codecs->len; i++)
     {
@@ -1029,6 +1047,8 @@ pass_local_codecs (GabbleMediaStream *stream,
       guint id, clock_rate, channels;
       gchar *name;
       GHashTable *params;
+      JingleCodec *c;
+      GValueArray *fb_codec;
 
       g_value_init (&codec, codec_struct_type);
       g_value_set_static_boxed (&codec, g_ptr_array_index (codecs, i));
@@ -1044,14 +1064,122 @@ pass_local_codecs (GabbleMediaStream *stream,
       c = jingle_media_rtp_codec_new (id, name,
           clock_rate, channels, params);
 
+      if (fbs != NULL)
+        {
+          fb_codec = g_hash_table_lookup (fbs, GUINT_TO_POINTER (id));
+          if (fb_codec != NULL)
+            {
+              if (G_VALUE_HOLDS_UINT (
+                      g_value_array_get_nth (fb_codec, 0)) &&
+                  G_VALUE_TYPE (g_value_array_get_nth (fb_codec, 1)) ==
+                  TP_ARRAY_TYPE_RTCP_FEEDBACK_MESSAGE_LIST)
+                {
+                  GValue *val;
+                  const GPtrArray *fb_array;
+                  guint j;
+
+                  val = g_value_array_get_nth (fb_codec, 0);
+                  c->trr_int = g_value_get_uint (val);
+
+                  val = g_value_array_get_nth (fb_codec, 1);
+                  fb_array = g_value_get_boxed (val);
+
+                  for (j = 0; j < fb_array->len; j++)
+                    {
+                      GValueArray *message = g_ptr_array_index (fb_array, j);
+                      const gchar *type;
+                      const gchar *subtype;
+
+                      val = g_value_array_get_nth (message, 0);
+                      type = g_value_get_string (val);
+
+                      val = g_value_array_get_nth (message, 1);
+                      subtype = g_value_get_string (val);
+
+                      c->feedback_msgs = g_list_append (c->feedback_msgs,
+                          jingle_feedback_message_new (type, subtype));
+                    }
+                }
+            }
+        }
       DEBUG ("adding codec %s (%u %u %u)", c->name, c->id, c->clockrate, c->channels);
-      li = g_list_append (li, c);
+      md->codecs = g_list_append (md->codecs, c);
       g_free (name);
       g_hash_table_unref (params);
     }
 
-  return jingle_media_rtp_set_local_codecs (
-      GABBLE_JINGLE_MEDIA_RTP (priv->content), li, ready, error);
+  if (fbs != NULL)
+    g_value_reset (&priv->local_feedback_messages);
+
+  hdrexts = g_value_get_boxed (&priv->local_rtp_hdrexts);
+
+  if (hdrexts != NULL)
+    {
+      gboolean have_initiator = FALSE;
+      gboolean initiated_by_us;
+
+      for (i = 0; i < hdrexts->len; i++)
+        {
+          GValueArray *hdrext;
+          guint id;
+          guint direction;
+          JingleContentSenders senders;
+          gchar *uri;
+          gchar *params;
+
+          hdrext = g_ptr_array_index (hdrexts, i);
+
+          g_assert (hdrext);
+          g_assert (hdrext->n_values == 4);
+          g_assert (G_VALUE_HOLDS_UINT   (g_value_array_get_nth (hdrext, 0)));
+          g_assert (G_VALUE_HOLDS_UINT   (g_value_array_get_nth (hdrext, 1)));
+          g_assert (G_VALUE_HOLDS_STRING (g_value_array_get_nth (hdrext, 2)));
+          g_assert (G_VALUE_HOLDS_STRING (g_value_array_get_nth (hdrext, 3)));
+
+          tp_value_array_unpack (hdrext, 4,
+              &id,
+              &direction,
+              &uri,
+              &params);
+
+          if (!have_initiator)
+            {
+              g_object_get (priv->content->session, "local-initiator",
+                  &initiated_by_us, NULL);
+              have_initiator = TRUE;
+            }
+
+          switch (direction)
+            {
+            case TP_MEDIA_STREAM_DIRECTION_BIDIRECTIONAL:
+              senders = JINGLE_CONTENT_SENDERS_BOTH;
+              break;
+            case TP_MEDIA_STREAM_DIRECTION_NONE:
+              senders = JINGLE_CONTENT_SENDERS_NONE;
+              break;
+            case TP_MEDIA_STREAM_DIRECTION_SEND:
+              senders = initiated_by_us ? JINGLE_CONTENT_SENDERS_INITIATOR :
+              JINGLE_CONTENT_SENDERS_RESPONDER;
+              break;
+            case TP_MEDIA_STREAM_DIRECTION_RECEIVE:
+              senders = initiated_by_us ? JINGLE_CONTENT_SENDERS_RESPONDER :
+              JINGLE_CONTENT_SENDERS_INITIATOR;
+              break;
+            default:
+              g_assert_not_reached ();
+            }
+
+          md->hdrexts = g_list_append (md->hdrexts,
+              jingle_rtp_header_extension_new (id, senders, uri));
+        }
+      /* Can only be used once */
+      g_value_reset (&priv->local_rtp_hdrexts);
+    }
+
+  jingle_media_description_simplify (md);
+
+  return jingle_media_rtp_set_local_media_description (
+      GABBLE_JINGLE_MEDIA_RTP (priv->content), md, ready, error);
 }
 
 /**
@@ -1066,10 +1194,16 @@ gabble_media_stream_set_local_codecs (TpSvcMediaStreamHandler *iface,
                                       DBusGMethodInvocation *context)
 {
   GabbleMediaStream *self = GABBLE_MEDIA_STREAM (iface);
+  GabbleMediaStreamPrivate *priv = self->priv;
   GError *error = NULL;
 
   DEBUG ("called");
 
+  if (codecs->len == 0)
+    goto done;
+
+  priv->local_codecs_set = TRUE;
+
   if (gabble_jingle_content_is_created_by_us (self->priv->content))
     {
       if (!pass_local_codecs (self, codecs, self->priv->created_locally,
@@ -1086,6 +1220,7 @@ gabble_media_stream_set_local_codecs (TpSvcMediaStreamHandler *iface,
     {
       DEBUG ("ignoring local codecs, waiting for codec intersection");
     }
+ done:
 
   tp_svc_media_stream_handler_return_from_set_local_codecs (context);
 }
@@ -1145,6 +1280,17 @@ gabble_media_stream_supported_codecs (TpSvcMediaStreamHandler *iface,
 
   DEBUG ("called");
 
+  if (codecs->len == 0)
+    {
+      GError e = { TP_ERRORS, TP_ERROR_INVALID_ARGUMENT,
+                   "SupportedCodecs must have a non-empty list of codecs" };
+
+      dbus_g_method_return_error (context, &e);
+      return;
+    }
+
+  priv->local_codecs_set = TRUE;
+
   if (priv->awaiting_intersection)
     {
       if (!pass_local_codecs (self, codecs, TRUE, &error))
@@ -1185,11 +1331,9 @@ gabble_media_stream_codecs_updated (TpSvcMediaStreamHandler *iface,
                                     DBusGMethodInvocation *context)
 {
   GabbleMediaStream *self = GABBLE_MEDIA_STREAM (iface);
-  gboolean codecs_set =
-      (g_value_get_boxed (&self->priv->native_codecs) != NULL);
   GError *error = NULL;
 
-  if (!codecs_set)
+  if (!self->priv->local_codecs_set)
     {
       GError e = { TP_ERRORS, TP_ERROR_NOT_AVAILABLE,
           "CodecsUpdated may only be called once an initial set of codecs "
@@ -1221,6 +1365,42 @@ gabble_media_stream_codecs_updated (TpSvcMediaStreamHandler *iface,
     }
 }
 
+/**
+ * gabble_media_stream_supported_header_extensions
+ *
+ * Implements D-Bus method SupportedHeaderExtensions
+ * on interface org.freedesktop.Telepathy.Media.StreamHandler
+ */
+static void
+gabble_media_stream_supported_header_extensions (TpSvcMediaStreamHandler *iface,
+                                                 const GPtrArray *hdrexts,
+                                                 DBusGMethodInvocation *context)
+{
+  GabbleMediaStream *self = GABBLE_MEDIA_STREAM (iface);
+
+  g_value_set_boxed (&self->priv->local_rtp_hdrexts, hdrexts);
+
+  tp_svc_media_stream_handler_return_from_supported_header_extensions (context);
+}
+
+/**
+ * gabble_media_stream_supported_feedback_messages
+ *
+ * Implements D-Bus method SupportedFeedbackMessages
+ * on interface org.freedesktop.Telepathy.Media.StreamHandler
+ */
+static void
+gabble_media_stream_supported_feedback_messages (TpSvcMediaStreamHandler *iface,
+                                                 GHashTable *messages,
+                                                 DBusGMethodInvocation *context)
+{
+  GabbleMediaStream *self = GABBLE_MEDIA_STREAM (iface);
+
+  g_value_set_boxed (&self->priv->local_feedback_messages, messages);
+
+  tp_svc_media_stream_handler_return_from_supported_feedback_messages (context);
+}
+
 void
 gabble_media_stream_close (GabbleMediaStream *stream)
 {
@@ -1238,13 +1418,31 @@ gabble_media_stream_close (GabbleMediaStream *stream)
 }
 
 static void
-new_remote_codecs_cb (GabbleJingleContent *content,
-    GList *clist, GabbleMediaStream *stream)
+insert_feedback_message (JingleFeedbackMessage *fb, GPtrArray *fb_msgs)
+{
+  GValueArray *msg;
+
+  msg = tp_value_array_build (3,
+      G_TYPE_STRING, fb->type,
+      G_TYPE_STRING, fb->subtype,
+      G_TYPE_STRING, "",
+      G_TYPE_INVALID);
+
+  g_ptr_array_add (fb_msgs, msg);
+}
+
+static void
+new_remote_media_description_cb (GabbleJingleContent *content,
+    JingleMediaDescription *md, GabbleMediaStream *stream)
 {
   GabbleMediaStreamPrivate *priv;
   GList *li;
   GPtrArray *codecs;
+  GPtrArray *hdrexts;
+  GHashTable *fbs;
   GType codec_struct_type = TP_STRUCT_TYPE_MEDIA_STREAM_HANDLER_CODEC;
+  gboolean have_initiator = FALSE;
+  gboolean initiated_by_us;
 
   DEBUG ("called");
 
@@ -1265,7 +1463,33 @@ new_remote_codecs_cb (GabbleJingleContent *content,
       g_value_take_boxed (&priv->remote_codecs, codecs);
     }
 
-  for (li = clist; li; li = li->next)
+  hdrexts = g_value_get_boxed (&priv->remote_rtp_hdrexts);
+
+  if (hdrexts->len != 0)
+    {
+      /* We already had some rtp hdrext; let's free the old list and make a new,
+       * empty one to fill in.
+       */
+      g_value_reset (&priv->remote_rtp_hdrexts);
+      hdrexts = dbus_g_type_specialized_construct (
+          TP_ARRAY_TYPE_RTP_HEADER_EXTENSIONS_LIST);
+      g_value_take_boxed (&priv->remote_rtp_hdrexts, hdrexts);
+    }
+
+  fbs = g_value_get_boxed (&priv->remote_feedback_messages);
+
+  if (g_hash_table_size (fbs) != 0)
+    {
+      /* We already had some rtp hdrext; let's free the old list and make a new,
+       * empty one to fill in.
+       */
+      g_value_reset (&priv->remote_feedback_messages);
+      fbs = dbus_g_type_specialized_construct (
+          TP_HASH_TYPE_RTCP_FEEDBACK_MESSAGE_MAP);
+      g_value_take_boxed (&priv->remote_feedback_messages, fbs);
+    }
+
+  for (li = md->codecs; li; li = li->next)
     {
       GValue codec = { 0, };
       JingleCodec *c = li->data;
@@ -1287,20 +1511,92 @@ new_remote_codecs_cb (GabbleJingleContent *content,
           5, c->params,
           G_MAXUINT);
 
+      if (md->trr_int != G_MAXUINT || c->trr_int != G_MAXUINT ||
+          md->feedback_msgs != NULL || c->feedback_msgs != NULL)
+        {
+          GValueArray *fb_msg_props;
+          guint trr_int;
+          GPtrArray *fb_msgs = g_ptr_array_new ();
+
+          if (c->trr_int != G_MAXUINT)
+            trr_int = c->trr_int;
+          else
+            trr_int = md->trr_int;
+
+          g_list_foreach (md->feedback_msgs, (GFunc) insert_feedback_message,
+              fb_msgs);
+          g_list_foreach (c->feedback_msgs, (GFunc) insert_feedback_message,
+              fb_msgs);
+
+          fb_msg_props = tp_value_array_build (2,
+              G_TYPE_UINT, trr_int,
+              TP_ARRAY_TYPE_RTCP_FEEDBACK_MESSAGE_LIST, fb_msgs,
+              G_TYPE_INVALID);
+
+          g_boxed_free (TP_ARRAY_TYPE_RTCP_FEEDBACK_MESSAGE_LIST, fb_msgs);
+
+          g_hash_table_insert (fbs, GUINT_TO_POINTER (c->id), fb_msg_props);
+        }
+
       g_ptr_array_add (codecs, g_value_get_boxed (&codec));
     }
 
+  for (li = md->hdrexts; li; li = li->next)
+    {
+      JingleRtpHeaderExtension *h = li->data;
+      TpMediaStreamDirection direction;
+
+      if (!have_initiator)
+        {
+          g_object_get (priv->content->session, "local-initiator",
+              &initiated_by_us, NULL);
+          have_initiator = TRUE;
+        }
+
+      switch (h->senders)
+        {
+        case JINGLE_CONTENT_SENDERS_BOTH:
+          direction = TP_MEDIA_STREAM_DIRECTION_BIDIRECTIONAL;
+          break;
+        case JINGLE_CONTENT_SENDERS_NONE:
+          direction = TP_MEDIA_STREAM_DIRECTION_NONE;
+          break;
+        case JINGLE_CONTENT_SENDERS_INITIATOR:
+          direction = initiated_by_us ? TP_MEDIA_STREAM_DIRECTION_SEND :
+          TP_MEDIA_STREAM_DIRECTION_RECEIVE;
+          break;
+        case JINGLE_CONTENT_SENDERS_RESPONDER:
+          direction = initiated_by_us ? TP_MEDIA_STREAM_DIRECTION_RECEIVE :
+          TP_MEDIA_STREAM_DIRECTION_SEND;
+          break;
+        default:
+          g_assert_not_reached ();
+        }
+
+      DEBUG ("new RTP header ext : %u %s", h->id, h->uri);
+
+      g_ptr_array_add (hdrexts,
+          tp_value_array_build (4,
+              G_TYPE_UINT,  h->id,
+              G_TYPE_UINT, direction,
+              G_TYPE_STRING, h->uri,
+              G_TYPE_STRING, "", /* No protocol defines parameters */
+              G_TYPE_INVALID));
+    }
+
   DEBUG ("pushing remote codecs");
 
-  push_remote_codecs (stream);
+  push_remote_media_description (stream);
 }
 
 
 static void
-push_remote_codecs (GabbleMediaStream *stream)
+push_remote_media_description (GabbleMediaStream *stream)
 {
   GabbleMediaStreamPrivate *priv;
   GPtrArray *codecs;
+  GPtrArray *hdrexts;
+  GHashTable *fbs;
 
   g_assert (GABBLE_IS_MEDIA_STREAM (stream));
 
@@ -1313,9 +1609,16 @@ push_remote_codecs (GabbleMediaStream *stream)
   if (codecs->len == 0)
     return;
 
+  hdrexts = g_value_get_boxed (&priv->remote_rtp_hdrexts);
+
+  fbs = g_value_get_boxed (&priv->remote_feedback_messages);
+
   DEBUG ("passing %d remote codecs to stream-engine",
                    codecs->len);
 
+  tp_svc_media_stream_handler_emit_set_remote_header_extensions (stream,
+      hdrexts);
+  tp_svc_media_stream_handler_emit_set_remote_feedback_messages (stream, fbs);
   tp_svc_media_stream_handler_emit_set_remote_codecs (stream, codecs);
 }
 
@@ -1352,7 +1655,7 @@ new_remote_candidates_cb (GabbleJingleContent *content,
           3, c->protocol == JINGLE_TRANSPORT_PROTOCOL_UDP ? 0 : 1,
           4, "RTP",
           5, "AVP",
-          6, c->preference,
+          6, (gdouble) (c->preference / 65536.0),
           7, c->type, /* FIXME: we're relying on 1:1 tp/jingle candidate type enums */
           8, c->username,
           9, c->password,
@@ -1726,116 +2029,8 @@ stream_handler_iface_init (gpointer g_iface, gpointer iface_data)
   IMPLEMENT(supported_codecs,);
   IMPLEMENT(unhold_failure,);
   IMPLEMENT(codecs_updated,);
-#undef IMPLEMENT
-}
-
-static void
-gabble_media_stream_props_get (TpSvcDBusProperties *iface,
-                               const gchar *interface_name,
-                               const gchar *property_name,
-                               DBusGMethodInvocation *context)
-{
-  GabbleMediaStream *self = GABBLE_MEDIA_STREAM (iface);
-  GValue value = { 0 };
-
-  if (!tp_strdiff (interface_name, TP_IFACE_MEDIA_STREAM_HANDLER))
-    {
-      if (!tp_strdiff (property_name, "RelayInfo"))
-        {
-          g_value_init (&value, TP_ARRAY_TYPE_STRING_VARIANT_MAP_LIST);
-          g_object_get_property ((GObject *) self, "relay-info", &value);
-        }
-      else if (!tp_strdiff (property_name, "STUNServers"))
-        {
-          /* FIXME: use correct macro when available */
-          g_value_init (&value, tp_type_dbus_array_su ());
-          g_object_get_property ((GObject *) self, "stun-servers", &value);
-        }
-      else if (!tp_strdiff (property_name, "NATTraversal"))
-        {
-          g_value_init (&value, G_TYPE_STRING);
-          g_object_get_property ((GObject *) self, "nat-traversal", &value);
-        }
-      else if (!tp_strdiff (property_name, "CreatedLocally"))
-        {
-          g_value_init (&value, G_TYPE_BOOLEAN);
-          g_object_get_property ((GObject *) self, "created-locally", &value);
-        }
-      else
-        {
-          GError not_implemented = { TP_ERRORS, TP_ERROR_NOT_IMPLEMENTED,
-              "Property not implemented" };
-
-          dbus_g_method_return_error (context, &not_implemented);
-          return;
-        }
-    }
-  else
-    {
-      GError not_implemented = { TP_ERRORS, TP_ERROR_NOT_IMPLEMENTED,
-          "Interface not implemented" };
-
-      dbus_g_method_return_error (context, &not_implemented);
-      return;
-    }
-
-  tp_svc_dbus_properties_return_from_get (context, &value);
-  g_value_unset (&value);
-}
-
-static void
-gabble_media_stream_props_get_all (TpSvcDBusProperties *iface,
-                                   const gchar *interface_name,
-                                   DBusGMethodInvocation *context)
-{
-  GabbleMediaStream *self = GABBLE_MEDIA_STREAM (iface);
-
-  if (!tp_strdiff (interface_name, TP_IFACE_MEDIA_STREAM_HANDLER))
-    {
-      GValue *value;
-      GHashTable *values = g_hash_table_new_full (g_str_hash, g_str_equal,
-          NULL, (GDestroyNotify) tp_g_value_slice_free);
-
-      value = tp_g_value_slice_new (TP_ARRAY_TYPE_STRING_VARIANT_MAP_LIST);
-      g_object_get_property ((GObject *) self, "relay-info", value);
-      g_hash_table_insert (values, "RelayInfo", value);
-
-      /* FIXME: use correct macro when available */
-      value = tp_g_value_slice_new (tp_type_dbus_array_su ());
-      g_object_get_property ((GObject *) self, "stun-servers", value);
-      g_hash_table_insert (values, "STUNServers", value);
-
-      value = tp_g_value_slice_new (G_TYPE_STRING);
-      g_object_get_property ((GObject *) self, "nat-traversal", value);
-      g_hash_table_insert (values, "NATTraversal", value);
-
-      value = tp_g_value_slice_new (G_TYPE_BOOLEAN);
-      g_object_get_property ((GObject *) self, "created-locally", value);
-      g_hash_table_insert (values, "CreatedLocally", value);
-
-      tp_svc_dbus_properties_return_from_get_all (context, values);
-      g_hash_table_destroy (values);
-    }
-  else
-    {
-      GError not_implemented = { TP_ERRORS, TP_ERROR_NOT_IMPLEMENTED,
-          "Interface not implemented" };
-
-      dbus_g_method_return_error (context, &not_implemented);
-    }
-}
-
-static void
-dbus_properties_iface_init (gpointer g_iface,
-                            gpointer iface_data G_GNUC_UNUSED)
-{
-  TpSvcDBusPropertiesClass *cls = g_iface;
-
-#define IMPLEMENT(x) \
-    tp_svc_dbus_properties_implement_##x (cls, gabble_media_stream_props_##x)
-  IMPLEMENT (get);
-  IMPLEMENT (get_all);
-  /* set not implemented in this class */
+  IMPLEMENT(supported_header_extensions,);
+  IMPLEMENT(supported_feedback_messages,);
 #undef IMPLEMENT
 }
 
@@ -1847,3 +2042,34 @@ gabble_media_stream_get_content (GabbleMediaStream *self)
    */
   return GABBLE_JINGLE_MEDIA_RTP (self->priv->content);
 }
+
+void
+gabble_media_stream_start_telephony_event (GabbleMediaStream *self,
+    guchar event)
+{
+  DEBUG ("stream %s: %c", self->name, tp_dtmf_event_to_char (event));
+
+  tp_svc_media_stream_handler_emit_start_telephony_event (
+      (TpSvcMediaStreamHandler *) self, event);
+}
+
+void
+gabble_media_stream_stop_telephony_event (GabbleMediaStream *self)
+{
+  DEBUG ("stream %s", self->name);
+
+  tp_svc_media_stream_handler_emit_stop_telephony_event (
+      (TpSvcMediaStreamHandler *) self);
+}
+
+void
+gabble_media_stream_add_dtmf_player (GabbleMediaStream *self,
+    TpDTMFPlayer *dtmf_player)
+{
+  tp_g_signal_connect_object (dtmf_player, "started-tone",
+      G_CALLBACK (gabble_media_stream_start_telephony_event), self,
+      G_CONNECT_SWAPPED);
+  tp_g_signal_connect_object (dtmf_player, "stopped-tone",
+      G_CALLBACK (gabble_media_stream_stop_telephony_event), self,
+      G_CONNECT_SWAPPED);
+}
