@@ -38,12 +38,14 @@
 #include <telepathy-glib/channel-manager.h>
 #include <telepathy-glib/intset.h>
 #include <wocky/wocky-caps-cache.h>
+#include <wocky/wocky-caps-hash.h>
+#include <wocky/wocky-disco-identity.h>
+#include <wocky/wocky-utils.h>
 
 #define DEBUG_FLAG GABBLE_DEBUG_PRESENCE
 
 #include "capabilities.h"
 #include "caps-channel-manager.h"
-#include "caps-hash.h"
 #include "conn-presence.h"
 #include "debug.h"
 #include "disco.h"
@@ -246,7 +248,7 @@ capability_info_free (GabbleCapabilityInfo *info)
       info->cap_set = NULL;
     }
 
-  gabble_disco_identity_array_free (info->identities);
+  wocky_disco_identity_array_free (info->identities);
   info->identities = NULL;
 
   tp_intset_destroy (info->guys);
@@ -1007,6 +1009,7 @@ _parse_node (GabblePresence *presence,
       DEBUG ("Client is Google Web Client");
 
       gabble_capability_set_add (cap_set, QUIRK_GOOGLE_WEBMAIL_CLIENT);
+      gabble_capability_set_add (cap_set, QUIRK_OMITS_CONTENT_CREATORS);
       gabble_presence_set_capabilities (presence, resource, cap_set, serial);
       gabble_capability_set_free (cap_set);
     }
@@ -1310,9 +1313,15 @@ _caps_disco_cb (GabbleDisco *disco,
     {
       gchar *computed_hash;
 
-      computed_hash = caps_hash_compute_from_lm_node (query_result);
+      computed_hash = wocky_caps_hash_compute_from_node (query_result);
 
-      if (g_str_equal (waiter_self->ver, computed_hash))
+      if (computed_hash == NULL)
+        {
+          DEBUG ("Unable to compute caps hash for '%s'.", jid);
+          trust = 0;
+          bad_hash = TRUE;
+        }
+      else if (g_str_equal (waiter_self->ver, computed_hash))
         {
           trust = capability_info_recvd (cache, node, handle, cap_set,
               CAPABILITY_BUNDLE_ENOUGH_TRUST, client_types);
@@ -1917,7 +1926,8 @@ gabble_presence_cache_maybe_remove (
   if (NULL == presence)
     return;
 
-  if (presence->status == GABBLE_PRESENCE_OFFLINE &&
+  if ((presence->status == GABBLE_PRESENCE_OFFLINE ||
+       presence->status == GABBLE_PRESENCE_UNKNOWN) &&
       presence->status_message == NULL &&
       !presence->keep_unavailable)
     {
@@ -1955,20 +1965,28 @@ gabble_presence_cache_do_update (
     gboolean *update_client_types)
 {
   GabblePresenceCachePrivate *priv = cache->priv;
-  TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
-      (TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_CONTACT);
-  const gchar *jid;
   GabblePresence *presence;
   GabbleCapabilitySet *old_cap_set;
   const GabbleCapabilitySet *new_cap_set;
   gboolean ret = FALSE;
 
-  jid = tp_handle_inspect (contact_repo, handle);
-  DEBUG ("%s (%d) resource %s prio %d presence %d message \"%s\"",
-      jid, handle,
-      resource == NULL ? "<null>" : resource,
-      priority, presence_id,
-      status_message == NULL ? "<null>" : status_message);
+  if (DEBUGGING)
+    {
+      TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
+          (TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_CONTACT);
+      const gchar *jid = tp_handle_inspect (contact_repo, handle);
+      const gchar *presence_name = wocky_enum_to_nick (
+          GABBLE_TYPE_PRESENCE_ID, presence_id);
+
+      if (presence_name == NULL)
+        presence_name = "plugin-specific, not an element of GabblePresenceId";
+
+      DEBUG ("%s (%d) resource %s prio %d presence %d (%s) message \"%s\"",
+          jid, handle,
+          resource == NULL ? "<null>" : resource,
+          priority, presence_id, presence_name,
+          status_message == NULL ? "<null>" : status_message);
+    }
 
   presence = gabble_presence_cache_get (cache, handle);
 
@@ -2104,8 +2122,12 @@ gabble_presence_cache_add_own_caps (
       gabble_capability_set_update (info->cap_set, cap_set);
     }
 
-  gabble_disco_identity_array_free (info->identities);
-  info->identities = gabble_disco_identity_array_copy (identities);
+  wocky_disco_identity_array_free (info->identities);
+
+  info->identities = NULL;
+
+  if (identities != NULL)
+    info->identities = wocky_disco_identity_array_copy (identities);
 
   info->complete = TRUE;
   info->trust = CAPABILITY_BUNDLE_ENOUGH_TRUST;
