@@ -502,8 +502,7 @@ start_stream_initiation (GabbleTubeStream *self,
           return FALSE;
         }
 
-      resource = gabble_presence_pick_resource_by_caps (presence,
-          DEVICE_AGNOSTIC,
+      resource = gabble_presence_pick_resource_by_caps (presence, 0,
           gabble_capability_set_predicate_has, NS_TUBES);
       if (resource == NULL)
         {
@@ -2252,8 +2251,7 @@ send_tube_offer (GabbleTubeStream *self,
       return FALSE;
     }
 
-  resource = gabble_presence_pick_resource_by_caps (presence,
-      DEVICE_AGNOSTIC,
+  resource = gabble_presence_pick_resource_by_caps (presence, 0,
       gabble_capability_set_predicate_has, NS_TUBES);
   if (resource == NULL)
     {
