@@ -13,13 +13,7 @@ import constants as cs
 import ns
 
 def run_test(q, bus, conn, stream):
-    conn.Connect()
-
-    _, initial_presence = q.expect_many(
-            EventPattern('dbus-signal', signal='StatusChanged',
-                args=[cs.CONN_STATUS_CONNECTED, cs.CSR_REQUESTED]),
-            EventPattern('stream-presence'),
-            )
+    initial_presence = q.expect('stream-presence')
 
     _, namespaces = disco_caps(q, stream, initial_presence)
 
