@@ -272,54 +272,54 @@ def test_call(q, bus, conn, stream,
             expected_stun_server, expected_stun_port)
 
     assertEquals(expected_relays, stream_props['RelayInfo'])
-    assertEquals(True, stream_props['RetrievedServerInfo'])
+    assertEquals(True, stream_props['HasServerInfo'])
 
 if __name__ == '__main__':
     # StreamedMedia tests
     exec_test(lambda q, b, c, s: test_streamed_media(q, b, c, s,
-        google=False))
+        google=False), do_connect=False)
     exec_test(lambda q, b, c, s: test_streamed_media(q, b, c, s,
         google=True, expected_stun_server='1.2.3.4', expected_stun_port=12345),
-        protocol=GoogleXmlStream)
+        protocol=GoogleXmlStream, do_connect=False)
     exec_test(lambda q, b, c, s: test_streamed_media(q, b, c, s,
         google=True, expected_stun_server='5.4.3.2', expected_stun_port=54321),
         protocol=GoogleXmlStream,
         params={'stun-server': 'resolves-to-5.4.3.2',
-            'stun-port': dbus.UInt16(54321)})
+            'stun-port': dbus.UInt16(54321)}, do_connect=False)
     exec_test(lambda q, b, c, s: test_streamed_media(q, b, c, s,
         google=True, expected_stun_server='1.2.3.4', expected_stun_port=12345),
         protocol=GoogleXmlStream,
         params={'fallback-stun-server': 'resolves-to-5.4.3.2',
-            'fallback-stun-port': dbus.UInt16(54321)})
+            'fallback-stun-port': dbus.UInt16(54321)}, do_connect=False)
     exec_test(lambda q, b, c, s: test_streamed_media(q, b, c, s,
         google=False, expected_stun_server='5.4.3.2', expected_stun_port=54321),
         params={'fallback-stun-server': 'resolves-to-5.4.3.2',
-            'fallback-stun-port': dbus.UInt16(54321)})
+            'fallback-stun-port': dbus.UInt16(54321)}, do_connect=False)
 
     # Call tests
     if CHANNEL_TYPE_CALL_ENABLED:
         exec_test(lambda q, b, c, s: test_call(q, b, c, s,
-            google=False))
+            google=False), do_connect=False)
         exec_test(lambda q, b, c, s: test_call(q, b, c, s,
             google=True, expected_stun_server='1.2.3.4',
             expected_stun_port=12345),
-            protocol=GoogleXmlStream)
+            protocol=GoogleXmlStream, do_connect=False)
         exec_test(lambda q, b, c, s: test_call(q, b, c, s,
             google=True, expected_stun_server='5.4.3.2',
             expected_stun_port=54321),
             protocol=GoogleXmlStream,
             params={'stun-server': 'resolves-to-5.4.3.2',
-                'stun-port': dbus.UInt16(54321)})
+                'stun-port': dbus.UInt16(54321)}, do_connect=False)
         exec_test(lambda q, b, c, s: test_call(q, b, c, s,
             google=True, expected_stun_server='1.2.3.4',
             expected_stun_port=12345),
             protocol=GoogleXmlStream,
             params={'fallback-stun-server': 'resolves-to-5.4.3.2',
-                'fallback-stun-port': dbus.UInt16(54321)})
+                'fallback-stun-port': dbus.UInt16(54321)}, do_connect=False)
         exec_test(lambda q, b, c, s: test_call(q, b, c, s,
             google=False, expected_stun_server='5.4.3.2',
             expected_stun_port=54321),
             params={'fallback-stun-server': 'resolves-to-5.4.3.2',
-                'fallback-stun-port': dbus.UInt16(54321)})
+                'fallback-stun-port': dbus.UInt16(54321)}, do_connect=False)
     else:
         print "NOTE: built with --disable-channel-type-call; omitting Call tests"
